<?php
	
//------------------------------------------------------------------------
// This script is a part of ABC Mail Manager
//
// Version: 2.5
// Homepage: www.zakkis.ca
// Copyright (c) 2002-2003 Zakkis Technology, Inc.
// All rights reserved.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
// REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGE.
//
//------------------------------------------------------------------------
	
class ArchiveLetter
{
	var $id;
	var $tm;
	var $type;
	var $subject;
	var $attachments = array();
	var $folder;

	function ArchiveLetter( $str )
	{
		$parts = split(':', trim( $str ) );
		
		$this->id = $parts[0];
		$this->tm = $parts[1];
		$this->type = $parts[2];
		$this->subject = $parts[3];
		$this->folder=$parts[4];
		
		// Collect attachments
		
		$i = 4;
		while( $i < count($parts) )
		{
			$name = $parts[ $i++ ];
			
			if( $i == count($parts) )
				break;
			
			$path = $parts[ $i++ ];
			
			$cur = count( $this->attachments );
			$this->attachments[ $cur ][ 0 ] = $name;
			$this->attachments[ $cur ][ 1 ] = $path;
		}
	}
	
	function HasAttachments()
	{
		return count( $this->attachments ) != 0;
	}
	
	function Delete()
	{
		foreach( $this->attachments as $file )
			unlink( $file[ 1 ] );
	}
	
} // end of archive letter

class ArchiveIndex
{

	/*
	 * Array of strings. Each string has format "$letter_id:$letter_type:$letter_subj"
	 * type is one of - 'htm', 'txt'
	 */
	
	var $data;
	var $folder;
	
	var $archive_dir = 'data/archive/';
	var $attachment_dir = 'data/archive/attach/';
	var $index_file = 'data/archive/index.dat';
	
	function ArchiveIndex( $reverse_order = false, $folder )
	{
		if( !file_exists( $this->index_file ) )
			return;
		
		$this->data = file( $this->index_file );
		
		foreach ( $this->data as $key=>$value) {
		
			$letter = $this->Get ( $key );
					
			if ( $letter->folder != $folder ) {
				
				unset ( $this->data[$key] );
				
			}
			
		}
		if( $reverse_order )
			$this->data = array_reverse( $this->data );
	}
	
	function Write()
	{
		$str = join( '', $this->data );
		$fd = fopen( $this->index_file, 'wb+' );
		fwrite( $fd, $str );
		fclose( $fd );
	}
	
	function Get( $index )
	{
		return new ArchiveLetter( $this->data[ $index ] );
	}
	
	function GetCount( $active_folder )
	{
		$count = 0;
		
		if ( isset ($this->data) ) {
			
			foreach ( $this->data as $data ) {
			
				$parts = split(':', trim( $data ) );
				$folder = $parts[4];
				
				if ( $folder == $active_folder )
					$count++;
				
			}
					
		}
				
		return $count;
	}
	
	function AddAttachments( $paths )
	{
		$result = "";

		for( $i = 0; $i < count($paths); $i++ )
		{
			$name = $paths[ $i ][ 0 ];
			$filename = $paths[ $i ][ 1 ];
			
			$path_parts = pathinfo( $name );
			
			$dest = $this->attachment_dir;
			$dest .= uniqid("");
			$dest .= '.';
			$dest .= $path_parts['extension'];
			
			if( copy( $filename, $dest ) )
				$result .= ":$name:$dest";
		}
		
		return $result;
	}
	
	function AddLetter( $type, $subj, $body, $paths, $folder )
	{
		$id = uniqid("");
		$filename = $this->archive_dir . $id . '.' . $type;
		
		$fd = fopen( $filename, 'wb+' );
		fwrite( $fd, $body );
		fclose( $fd );
		
		$str = $this->AddAttachments( $paths );	
	
		$entry = $id . ':' . time() . ':' . $type . ':' . $subj . ':' . $folder;
		if( $str != "" )
			$entry .= $str;
			
		$entry .= "\n";
		
		$this->data[] = $entry;
	}
	
	function DeleteLetter( $index )
	{
		$item = $this->Get( $index );
		$item->Delete();
	
		$filename = $this->archive_dir . $item->id . '.' . $item->type;
		@unlink( $filename );
		
		unset( $this->data[ $index ] );
	}
	
	function DeleteFolderLetters( $folder )
	{
		foreach ( $this->data as $key=>$value) {
		
			$letter = $this->Get ( $key );
			if ( $letter->folder == $folder ) {
				
				$this->DeleteLetter ( $key );
				unset ( $this->data[$key] );
				$this->Write();
			}
			
		}
		
	}
	
	function ChangeFolder( $oldfolder, $newfolder )
	{
			
		foreach ( $this->data as $key=>$value) {
							
			$parts = split(':', trim( $value ) );
						
			if ( $parts[4] == $oldfolder ) {
				
				$parts[4] = $newfolder;
				$entry = $parts[0] . ':' . time() . ':' . $parts[2] . ':' . $parts[3] . ':' . $parts[4];
				$entry .= "\n";
				$this -> data[$key] = $entry;
				
				$this->Write();
								
			}
			
			
		}
		
		
	}
	
} // end of ArchiveIndex

?>